@@ -2,7 +2,7 @@ package openapi3
 
 import (
 	"context"
-	"io/ioutil"
+	"os"
 	"regexp"
 	"testing"
 
@@ -41,25 +41,25 @@ func TestInternalizeRefs(t *testing.T) {
 			err = doc.Validate(ctx)
 			require.NoError(t, err, "validating internalized spec")
 
-			data, err := doc.MarshalJSON()
+			actual, err := doc.MarshalJSON()
 			require.NoError(t, err, "marshalling internalized spec")
 
 			// run a static check over the file, making sure each occurence of a
 			// reference is followed by a #
-			numRefs := len(regexpRef.FindAll(data, -1))
-			numInternalRefs := len(regexpRefInternal.FindAll(data, -1))
+			numRefs := len(regexpRef.FindAll(actual, -1))
+			numInternalRefs := len(regexpRefInternal.FindAll(actual, -1))
 			require.Equal(t, numRefs, numInternalRefs, "checking all references are internal")
 
-			// load from data, but with the path set to the current directory
-			doc2, err := sl.LoadFromData(data)
+			// load from actual, but with the path set to the current directory
+			doc2, err := sl.LoadFromData(actual)
 			require.NoError(t, err, "reloading spec")
 			err = doc2.Validate(ctx)
 			require.NoError(t, err, "validating reloaded spec")
 
 			// compare with expected
-			data0, err := ioutil.ReadFile(test.filename + ".internalized.yml")
+			expected, err := os.ReadFile(test.filename + ".internalized.yml")
 			require.NoError(t, err)
-			require.JSONEq(t, string(data), string(data0))
+			require.JSONEq(t, string(expected), string(actual))
 		})
 	}
 }
