@@ -11,7 +11,7 @@ import (
 // T is the root of an OpenAPI v3 document
 // See https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.3.md#oasObject
 type T struct {
-	ExtensionProps
+	ExtensionProps `json:"-" yaml:"-"`
 
 	OpenAPI      string               `json:"openapi" yaml:"openapi"` // Required
 	Components   Components           `json:"components,omitempty" yaml:"components,omitempty"`
@@ -66,70 +66,57 @@ func (doc *T) Validate(ctx context.Context, opts ...ValidationOption) error {
 		return errors.New("value of openapi must be a non-empty string")
 	}
 
+	var wrap func(error) error
 	// NOTE: only mention info/components/paths/... key in this func's errors.
 
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid components: %w", e) }
-		if err := doc.Components.Validate(ctx); err != nil {
+	wrap = func(e error) error { return fmt.Errorf("invalid components: %w", e) }
+	if err := doc.Components.Validate(ctx); err != nil {
+		return wrap(err)
+	}
+
+	wrap = func(e error) error { return fmt.Errorf("invalid info: %w", e) }
+	if v := doc.Info; v != nil {
+		if err := v.Validate(ctx); err != nil {
+			return wrap(err)
+		}
+	} else {
+		return wrap(errors.New("must be an object"))
+	}
+
+	wrap = func(e error) error { return fmt.Errorf("invalid paths: %w", e) }
+	if v := doc.Paths; v != nil {
+		if err := v.Validate(ctx); err != nil {
+			return wrap(err)
+		}
+	} else {
+		return wrap(errors.New("must be an object"))
+	}
+
+	wrap = func(e error) error { return fmt.Errorf("invalid security: %w", e) }
+	if v := doc.Security; v != nil {
+		if err := v.Validate(ctx); err != nil {
 			return wrap(err)
 		}
 	}
 
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid info: %w", e) }
-		if v := doc.Info; v != nil {
-			if err := v.Validate(ctx); err != nil {
-				return wrap(err)
-			}
-		} else {
-			return wrap(errors.New("must be an object"))
+	wrap = func(e error) error { return fmt.Errorf("invalid servers: %w", e) }
+	if v := doc.Servers; v != nil {
+		if err := v.Validate(ctx); err != nil {
+			return wrap(err)
 		}
 	}
 
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid paths: %w", e) }
-		if v := doc.Paths; v != nil {
-			if err := v.Validate(ctx); err != nil {
-				return wrap(err)
-			}
-		} else {
-			return wrap(errors.New("must be an object"))
+	wrap = func(e error) error { return fmt.Errorf("invalid tags: %w", e) }
+	if v := doc.Tags; v != nil {
+		if err := v.Validate(ctx); err != nil {
+			return wrap(err)
 		}
 	}
 
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid security: %w", e) }
-		if v := doc.Security; v != nil {
-			if err := v.Validate(ctx); err != nil {
-				return wrap(err)
-			}
-		}
-	}
-
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid servers: %w", e) }
-		if v := doc.Servers; v != nil {
-			if err := v.Validate(ctx); err != nil {
-				return wrap(err)
-			}
-		}
-	}
-
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid tags: %w", e) }
-		if v := doc.Tags; v != nil {
-			if err := v.Validate(ctx); err != nil {
-				return wrap(err)
-			}
-		}
-	}
-
-	{
-		wrap := func(e error) error { return fmt.Errorf("invalid external docs: %w", e) }
-		if v := doc.ExternalDocs; v != nil {
-			if err := v.Validate(ctx); err != nil {
-				return wrap(err)
-			}
+	wrap = func(e error) error { return fmt.Errorf("invalid external docs: %w", e) }
+	if v := doc.ExternalDocs; v != nil {
+		if err := v.Validate(ctx); err != nil {
+			return wrap(err)
 		}
 	}
 
