@@ -4,8 +4,9 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/getkin/kin-openapi/openapi3"
 	"github.com/stretchr/testify/require"
+
+	"github.com/getkin/kin-openapi/openapi3"
 )
 
 func TestRegisterArrayUniqueItemsChecker(t *testing.T) {
@@ -32,5 +33,5 @@ func TestRegisterArrayUniqueItemsChecker(t *testing.T) {
 
 	err = schema.VisitJSON(val)
 	require.Error(t, err)
-	require.True(t, strings.HasPrefix(err.Error(), "Duplicate items found"))
+	require.True(t, strings.HasPrefix(err.Error(), "duplicate items found"))
 }
