@@ -34,6 +34,8 @@ func (enc *ValidationErrorEncoder) Encode(ctx context.Context, err error, w http
 		cErr = convertBasicRequestError(e)
 	} else if e.Err == ErrInvalidRequired {
 		cErr = convertErrInvalidRequired(e)
+	} else if e.Err == ErrInvalidEmptyValue {
+		cErr = convertErrInvalidEmptyValue(e)
 	} else if innerErr, ok := e.Err.(*ParseError); ok {
 		cErr = convertParseError(e, innerErr)
 	} else if innerErr, ok := e.Err.(*openapi3.SchemaError); ok {
@@ -87,6 +89,19 @@ func convertErrInvalidRequired(e *RequestError) *ValidationError {
 	}
 }
 
+func convertErrInvalidEmptyValue(e *RequestError) *ValidationError {
+	if e.Err == ErrInvalidEmptyValue && e.Parameter != nil {
+		return &ValidationError{
+			Status: http.StatusBadRequest,
+			Title:  fmt.Sprintf("parameter %q in %s is not allowed to be empty", e.Parameter.Name, e.Parameter.In),
+		}
+	}
+	return &ValidationError{
+		Status: http.StatusBadRequest,
+		Title:  e.Error(),
+	}
+}
+
 func convertParseError(e *RequestError, innerErr *ParseError) *ValidationError {
 	// We treat path params of the wrong type like a 404 instead of a 400
 	if innerErr.Kind == KindInvalidFormat && e.Parameter != nil && e.Parameter.In == "path" {
